@@ -20,6 +20,11 @@ ADDITIONAL_PROPERTIES = "_additionalProperties"
 IS_IMMUTABLE = "_immutable"
 OPTIONAL_FIELDS = "_optional"
 
+py_version = sys.version_info[0:2]
+python_ver_36 = py_version == (3, 6)
+python_ver_atleast_than_37 = py_version > (3, 6)
+python_ver_atleast_39 = py_version >= (3, 9)
+
 
 class ImmutableMixin:
     _field_definition = None
@@ -127,6 +132,8 @@ class _FieldMeta(type):
             return ClassReference(val)
         elif is_function_returning_field(val):
             return val()
+        elif val is None:
+            return NoneField()
         else:
 
             def get_state(value):
@@ -384,28 +391,36 @@ def convert_basic_types(v):
     return type_mapping.get(v, None)
 
 
-def get_typing_lib_info(v):
-    from .fields import AnyOf
-
+def _type_is_generic(v):
     class Foo:
         pass
 
-    py_version = sys.version_info[0:2]
-    python_ver_atleast_than_37 = py_version >= (3, 6)
-    python_ver_atleast_39 = py_version >= (3, 9)
-
+    origin = getattr(v, "__origin__", None)
+    typing_base = getattr(typing, "_TypingBase", Foo)
     generic_alias = getattr(typing, "_GenericAlias", Foo)
     special_generic_alias = getattr(typing, "_SpecialGenericAlias", Foo)
-    origin = getattr(v, "__origin__", None)
-    is_typing_generic = (
-        python_ver_atleast_than_37
-        and isinstance(v, (generic_alias, special_generic_alias))
-    ) or (
-        python_ver_atleast_39
-        and origin in {list, dict, tuple, set, frozenset, typing.Union}
+    return (
+        (python_ver_36 and isinstance(v, typing_base))
+        or (
+            python_ver_atleast_than_37
+            and isinstance(v, (generic_alias, special_generic_alias))
+        )
+        or (
+            python_ver_atleast_39
+            and origin in {list, dict, tuple, set, frozenset, typing.Union}
+        )
     )
-    if not is_typing_generic:
+
+
+def get_typing_lib_info(v):
+    from .fields import AnyOf
+
+    if v == type(None):
+        return NoneField()
+
+    if not _type_is_generic(v):
         return convert_basic_types(v)
+    origin = getattr(v, "__origin__", None)
     mapped_type = convert_basic_types(origin)
     if mapped_type is None:
         raise TypeError("{} type is not supported".format(v))
@@ -416,7 +431,15 @@ def get_typing_lib_info(v):
         get_typing_lib_info(a) for a in args_raw if not isinstance(a, typing.TypeVar)
     ]
     if not all(mapped_args):
-        raise TypeError("invalid type {}".format(v))
+        if mapped_type == AnyOf:
+            for i, arg in enumerate(mapped_args):
+                if arg is None:
+                    if isinstance(args_raw[i], type):
+                        mapped_args[i] = Field[args_raw[i]]
+                    else:
+                        raise TypeError("invalid type {}".format(v))
+        else:
+            raise TypeError("invalid type {}".format(v))
     if mapped_args:
         if mapped_type == AnyOf:
             return mapped_type(fields=mapped_args)
@@ -429,6 +452,7 @@ def get_typing_lib_info(v):
 def add_annotations_to_class_dict(cls_dict):
     annotations = cls_dict.get("__annotations__", {})
     defaults = {}
+    optional_fields = set(cls_dict.get(OPTIONAL_FIELDS, []))
     if isinstance(annotations, dict):
         for k, v in annotations.items():
             first_arg = getattr(v, "__args__", [0])[0]
@@ -440,13 +464,25 @@ def add_annotations_to_class_dict(cls_dict):
             else:
                 the_type = get_typing_lib_info(v)
                 if the_type:
+                    from .fields import AnyOf
+
+                    if isinstance(the_type, AnyOf) and getattr(
+                        the_type, "_is_optional", False
+                    ):
+                        optional_fields.add(k)
                     if k in cls_dict:
+                        default = cls_dict[k]
                         try:
-                            the_type = the_type(default=cls_dict[k])
+                            if isinstance(the_type, Field):
+                                the_type._try_default_value(default)
+                            else:
+                                the_type = the_type(default=default)
                         except Exception as e:
                             raise e.__class__("{}: {}".format(k, str(e))) from e
                         defaults[k] = cls_dict[k]
                     cls_dict[k] = the_type
+    if optional_fields:
+        cls_dict[OPTIONAL_FIELDS] = optional_fields
     cls_dict[DEFAULTS] = defaults
 
 
@@ -662,10 +698,12 @@ class ImmutableStructure(Structure):
 
         b = B(y = 3, z = [1,2,3], m = {'a': 1, 'b': 2})
 
-        # each of the following lines will raise an exception:
+        # any of the following lines will raise an exception:
         b.y = 1
         b.z[1] += 1
         b.m['c'] = 4
+        b.z.clear()
+        b.m.pop('a')
 
     """
 
@@ -695,6 +733,26 @@ class TypedField(Field):
         super().__set__(instance, value)
 
 
+class NoneField(TypedField):
+    """
+    A field that maps to a single allowable value: None.
+    This is useful to define optional fields or optional values such as:
+
+    .. code-block:: python
+
+       class Foo(Structure):
+           optional_1: typing.Optional[Array]     # NoneField is used implicitly
+           optional_2: AnyOf[Array, NoneField]
+           optional_3: AnyOf[Array, None]         # the conversion from None to NoneField is implicit
+
+           arr_maybe_int_1: Array[AnyOf[Integer, NoneField]]
+           arr_maybe_int_2: Array[AnyOf[Integer, None]]    # the conversion from None to NoneField is implicit
+
+    """
+
+    _ty = type(None)
+
+
 class ValidatedTypedField(TypedField):
     def __set__(self, instance, value):
         self._validate_func(value)  # pylint: disable=E1101
